@@ -29,7 +29,7 @@ class FontSelector(Sensor):
         Sensor.__init__(self)
         self._watch_config(self.config_watcher)
         # setup configuration entries
-        if (len(fontargs) == 1 and "  " in fontargs[0]):
+        if (len(fontargs) == 1 and fontargs[0].find("  ") != -1):
             fontargs = \
                (fontargs[0].replace("   ", ",").replace("  ", ",")).split(",")
             print "Using spaces to separate font entries is deprecated."
