@@ -1,4 +1,8 @@
 import desktop
+try:
+    from utils import _ewmh as ewmh
+except:
+    print "Couldn't load EWMH support. Some window flags will not work."
 
 import gtk
 
@@ -26,6 +30,8 @@ class GlassWindow(gtk.Window):
     #
     def __init__(self, wintype = gtk.WINDOW_TOPLEVEL):
 
+        self.__gc_lock = 0
+
         # the transparency mode to use
         self.__transparency = self.TRANSPARENCY_PSEUDO
 
@@ -36,11 +42,6 @@ class GlassWindow(gtk.Window):
         self.__position = (0, 0)
         self.__size = (0, 0)
 
-        # whether the window is below all others
-        self.__is_lower = 0
-        self.__lowering_lock = 0
-
-        self.__is_above = 0
 
         gtk.Window.__init__(self, wintype)
         self.connect("configure-event", self.__on_configure)
@@ -49,40 +50,6 @@ class GlassWindow(gtk.Window):
         self.__bg_image = gtk.gdk.Pixbuf(0, 1, 8, 10, 10)
         self.__bg_image.fill(0xffffff)
 
-        
-        self.connect("focus-in-event", self.__lower)
-        self.connect("button-press-event", self.__lower)
-        #self.connect("expose-event", self.__lower) # caused trouble with XFCE4
-
-
-
-    def __lower(self, src, event):
-
-        if (self.__lowering_lock): return
-        
-        if (self.__is_lower and self.window):
-            self.window.lower()
-
-
-        # lock lowering for some time to avoid CPU intensive flickering
-        self.__lowering_lock = 1
-        def f(self): self.__lowering_lock = 0
-        gtk.timeout_add(25, f, self)
-
-
-
-    def __raise(self, src, event):
-
-        x, y = self.get_pointer()
-        w, h = self.size_request()
-        
-        if (self.window):
-            self.window.raise_()
-
-        if (self.__is_above):
-            gtk.timeout_add(100, self.__raise, None, None)
-
-
 
 
     #
@@ -101,20 +68,6 @@ class GlassWindow(gtk.Window):
     #
     def set_window_type(self, wtype):
 
-        self.__is_lower = 0
-        self.__is_above = 0
-
-
-        if (wtype & self.TYPE_KEEP_BELOW):
-            self.__is_lower = 1
-            self.__lower(None, None)
-        elif (wtype & self.TYPE_KEEP_ABOVE):
-            self.__is_above = 1
-            self.__raise(None, None)
-        #else:
-        #    pass
-
-
         if (wtype & self.TYPE_STICKY):
             self.stick()
         else:
@@ -125,12 +78,25 @@ class GlassWindow(gtk.Window):
         else:
             self.set_decorated(gtk.FALSE)
 
+        # do it later, if the window isn't currently visible
+        if (not self.get_property("visible")):
+            gtk.timeout_add(100, self.set_window_type, wtype)
+            return
+
+            
+        if (wtype & self.TYPE_KEEP_BELOW):
+            ewmh.set_below(self.window)
+
+        elif (wtype & self.TYPE_KEEP_ABOVE):
+            ewmh.set_above(self.window)
+
+
 
 
     #
     # Sets the transparency mode of this window.
     #
-    def set_transparency_mode(self, mode):
+    def set_transparency_mode(self, mode, width, height):
 
         self.__transparency = mode
 
@@ -150,25 +116,31 @@ class GlassWindow(gtk.Window):
         width = self.__bg_root.get_width()
         height = self.__bg_root.get_height()
 
-        # we cannot use Pixbuf.copy() here because of a memory leak
-        # bug in pygtk versions < 1.99.14
-        image = gtk.gdk.Pixbuf(0, 1, 8, width, height)
-        self.__bg_root.copy_area(0, 0, width, height, image, 0, 0)
-
-
         # set as window bg
-        pix, mask = image.render_pixmap_and_mask()
+        pix, mask = self.__bg_root.render_pixmap_and_mask()
 
         style = gtk.Style()
         style.bg_pixmap[gtk.STATE_NORMAL] = pix
         self.set_style(style)
 
+        # work around the memory leak bug #110261 of pygtk
+        if (not self.__gc_lock):
+            self.__gc_lock = 1
+
+            def f(self):
+                self.__gc_lock = 0
+                import gc; gc.collect()
+
+            gtk.timeout_add(1000, f, self)
+
+        #end if
+
+
         # tell gtk to redraw this window ASAP
         self.queue_draw()
 
 
 
-
     #
     # Updates the background for transparency.
     #
@@ -190,6 +162,7 @@ class GlassWindow(gtk.Window):
     #
     def __capture_bg(self, x, y, width, height, noscreenshot = 0):
 
+        #del self.__bg_root
         # try capturing the background
         if (self.__transparency == self.TRANSPARENCY_PSEUDO):
             try:
