@@ -48,7 +48,6 @@ class DataTarget(Observable):
         del self.__actions
         del self.__parent
         del self.__display
-        print "forget", self
 
 
     #
@@ -97,7 +96,7 @@ class DataTarget(Observable):
 
         commands = actionparser.parse(call)
         if (not self.__actions.has_key(action)): self.__actions[action] = []
-        self.__actions[action] += commands
+        self.__actions[action] = commands
 
 
 
