@@ -1,9 +1,9 @@
-from desktop.GlassWindow import GlassWindow
+from DisplayContainer import DisplayContainer
 from TargetGroup import TargetGroup
 from DisplayTarget import DisplayTarget
 from utils.Observable import Observable
 from DisplayConfigurator import DisplayConfigurator
-from main import ICON, DEFAULT_SENSOR
+from main import DEFAULT_SENSOR, UNSET_COORD
 from main import admin
 from utils import vfs
 
@@ -14,22 +14,20 @@ import os
 #
 # Class for display windows.
 #
-class Display(GlassWindow, Observable):
+class Display(Observable, gtk.EventBox):
 
     # observer commands
     OBS_CLOSE = 0
     OBS_RESTART = 1
 
-    # mapping: str -> window type
-    __WINDOW_FLAGS = {"below": GlassWindow.TYPE_KEEP_BELOW,
-                      "above": GlassWindow.TYPE_KEEP_ABOVE,
-                      "sticky": GlassWindow.TYPE_STICKY,
-                      "managed": GlassWindow.TYPE_MANAGED}
-
-
 
     def __init__(self, id):
 
+        # the container of this display; we use a proxy until we get
+        # the real thing; the proxy remembers set values and passes them on
+        # to the real container
+        self.__container = DisplayContainer(self)
+
         # the path of the .display file
         self.__path = vfs.getcwd()
 
@@ -46,18 +44,6 @@ class Display(GlassWindow, Observable):
         # unwanted events)
         self.__last_pointer_pos = (-1, -1)
 
-        # timeout flag for mouse buttons
-        self.__button_timeout = 1
-
-        # timeout flag for menu detection
-        self.__menu_timeout = 0
-
-        # window position for detecting moves
-        self.__window_pos = (-1, -1)
-
-        # window size for detecting resizing
-        self.__window_size = (0, 0)
-
         # mapping between sensors and targets; which target watches
         # which sensor?
         # (sensor, port) -> (target, property)
@@ -66,42 +52,43 @@ class Display(GlassWindow, Observable):
         # temporary data for remembering the position of the last mouse click
         self.__pointer_pos = (0, 0)
 
-        # temporary data used for dragging windows
-        self.__is_dragging = 0
-        self.__drag_offset = (0, 0)
+        # whether the display reacts on events
+        self.__is_sensitive = gtk.TRUE
 
+        gtk.EventBox.__init__(self)
+        self.show()
 
-        GlassWindow.__init__(self, gtk.WINDOW_TOPLEVEL)
-        self.set_size_request(-1, -1)
-
-        self.__ebox = gtk.EventBox()
-        self.__ebox.show()
-        self.add(self.__ebox)
-
-        # set the icon
-        self.set_icon(gtk.gdk.pixbuf_new_from_file(ICON))
         # set up event handlers
-        #self.connect("configure-event", self.__on_configure)
-        self.__ebox.connect("button-press-event", self.__on_button, 0)
-        self.__ebox.connect("button-release-event", self.__on_button, 1)
-        self.__ebox.connect("scroll-event", self.__on_scroll)
-        self.__ebox.connect("motion-notify-event", self.__on_motion, 0)
-        self.__ebox.connect("leave-notify-event", self.__on_motion, 1)
-        self.__ebox.connect("delete-event", self.__on_close)
-        self.__ebox.add_events(gtk.gdk.BUTTON_PRESS_MASK |
+        self.connect("button-press-event", self.__on_button, 0)
+        self.connect("button-release-event", self.__on_button, 1)
+        self.connect("scroll-event", self.__on_scroll)
+        self.connect("motion-notify-event", self.__on_motion, 0)
+        self.connect("leave-notify-event", self.__on_motion, 1)
+        self.connect("delete-event", self.__on_close)
+        self.add_events(gtk.gdk.BUTTON_PRESS_MASK |
                         gtk.gdk.BUTTON_RELEASE_MASK |
                         gtk.gdk.LEAVE_NOTIFY_MASK |
                         gtk.gdk.POINTER_MOTION_MASK)
 
 
-        # sawfish needs this
-        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
-        self.realize()
-        self.set_property("skip-taskbar-hint", 1)
-        self.set_property("skip-pager-hint", 1)
 
-        self.set_window_type(self.TYPE_STICKY | self.TYPE_KEEP_BELOW)
+    #
+    # Sets the container of the display.
+    #
+    def set_container(self, container):
 
+        # pass the values stored by the proxy on to the real container
+        self.__container.pass_values(container)
+        self.__container = container
+
+
+    #
+    # Sets the sensitive flag of the display. Insensitive displays don't react
+    # on user events.
+    #
+    def set_sensitive(self, value):
+
+        self.__is_sensitive = value
 
 
     #
@@ -117,9 +104,9 @@ class Display(GlassWindow, Observable):
 
         # create the root TargetGroup
         self.__group = TargetGroup(None, self)
-        self.__group.set_position(-1, -1)
+        self.__group.set_position(UNSET_COORD, UNSET_COORD)
         self.__group.add_children(childrendata)
-        self.__ebox.add(self.__group)
+        self.add(self.__group)
         self.__group.add_observer(self.__on_observe_group)
 
 
@@ -136,7 +123,7 @@ class Display(GlassWindow, Observable):
             configurators.append(s.get_configurator())
 
         dconf = DisplayConfigurator(configurators)
-        dconf.set_transient_for(self)
+        dconf.set_transient_for(self.get_toplevel())
 
 
     #
@@ -156,24 +143,7 @@ class Display(GlassWindow, Observable):
         self.__group.delete()
         self.__group.destroy()
         del self.__group
-        self.destroy()
-
-
-
-    #
-    # Reacts on configuring the window.
-    #
-    def __on_configure(self, src, event):
-
-        width = event.width
-        height = event.height
-
-        if ((width, height) != self.__window_size):
-            self.__window_size = (width, height)
-            self.set_config("width", width)
-            self.set_config("height", height)
-        #end if
-
+        self.__container.close()
 
 
     #
@@ -186,24 +156,6 @@ class Display(GlassWindow, Observable):
 
 
 
-    #
-    # Reacts on moving the mouse.
-    #
-    def __on_move(self):
-
-        if (self.__is_dragging):
-            offx, offy = self.__drag_offset
-            winx, winy = self.get_position()
-            x, y = self.get_pointer()
-            x += winx; y += winy
-            self.move(x - offx, y - offy)
-
-            return gtk.TRUE
-
-        else:
-            return gtk.FALSE
-
-
 
     def file_drop(self, files, x, y):
         ''' Sends the call with path and files to the sensor '''
@@ -215,15 +167,11 @@ class Display(GlassWindow, Observable):
         for t, path in rtargets:
             if (t.has_action(DisplayTarget.ACTION_FILE_DROP)):
                 call = t.get_action_call(DisplayTarget.ACTION_FILE_DROP)
-                #index = targets.index(t)
-                #targetpath = path[:index + 1]
                 print files, t, path
-                self.__call_sensor(call, path, files, x, y)
+                self._call_sensor(call, path, files, x, y)
                 break
         #end for
 
-        #self.__call_sensor(call, path, files, x, y)
-
 
 
     #
@@ -231,25 +179,8 @@ class Display(GlassWindow, Observable):
     #
     def __on_button(self, src, event, is_release = 0):
 
-        # local timeout function
-        def do_timeout(self): self.__button_timeout = 1
-
-        # function for detecting holding down the mouse long enough for menu
-        # TODO: this is obsolete, isn't it?
-        def do_menu(self, targets, path):
-            if (not self.__menu_timeout): return
-            targets.reverse()
-            for t in targets:
-                if (t.has_action(t.ACTION_MENU)):
-                    call = t.get_action_call(t.ACTION_MENU)
-                    index = targets.index(t)
-                    targetpath = path[:index + 1]
-                    self.__call_sensor(call, targetpath, button)
-                    break
-            #end for
-        #end def
-
-        self.__menu_timeout = 0
+        if (not self.__is_sensitive): return
+        
         px, py = self.get_pointer()
         lx, ly = self.__pointer_pos
         button = event.button
@@ -264,11 +195,6 @@ class Display(GlassWindow, Observable):
                 action = DisplayTarget.ACTION_PRESS
             elif (button == 2):
                 #action = DisplayTarget.ACTION_PRESS
-                x, y = self.get_pointer()
-                self.__is_dragging = 1
-                self.__drag_offset = (x, y)
-                self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.FLEUR))
-                gtk.timeout_add(20, self.__on_move)
                 return
             elif (button == 3):
                 return
@@ -283,8 +209,6 @@ class Display(GlassWindow, Observable):
                     action = DisplayTarget.ACTION_RELEASE
             elif (button == 2):
                 #action = DisplayTarget.ACTION_RELEASE
-                self.__is_dragging = 0
-                self.window.set_cursor(None)
                 return
             elif (button == 3):
                 action = DisplayTarget.ACTION_MENU
@@ -298,16 +222,14 @@ class Display(GlassWindow, Observable):
         for t, path in targets:
             if (t.has_action(action)):
                 call = t.get_action_call(action)
-                #index = targets.index(t)
-                #targetpath = path[:index + 1]
-                self.__call_sensor(call, path, button)
+                self._call_sensor(call, path, button)
                 called = 1
                 break
         #end for
 
         # make sure that there is always a popup menu
         if (action == DisplayTarget.ACTION_MENU and not called):
-            self.__call_sensor([(DEFAULT_SENSOR, "menu", [])], [], 3)
+            self._call_sensor([(DEFAULT_SENSOR, "menu", [])], [], 3)
 
 
 
@@ -316,6 +238,8 @@ class Display(GlassWindow, Observable):
     #
     def __on_motion(self, src, event, is_leave):
 
+        if (not self.__is_sensitive): return
+
         px, py = self.get_pointer()
         if ((px, py) == self.__last_pointer_pos): return
         else: self.__last_pointer_pos = (px, py)
@@ -339,42 +263,23 @@ class Display(GlassWindow, Observable):
             # enter
             if (not (t, path) in self.__last_targets and
                 t.has_action(t.ACTION_ENTER)):
-                #index = targets.index(t)
-                #targetpath = path[:index + 1]
-                self.__call_sensor(t.get_action_call(t.ACTION_ENTER), path)
+                self._call_sensor(t.get_action_call(t.ACTION_ENTER), path)
 
             # leave
             elif (not (t, path) in targets and t.has_action(t.ACTION_LEAVE)):
-                #index = self.__last_targets.index(t)
-                #targetpath = self.__last_target_path[:index + 1]
-                self.__call_sensor(t.get_action_call(t.ACTION_LEAVE), path)
+                self._call_sensor(t.get_action_call(t.ACTION_LEAVE), path)
 
 
             # motion
             elif ((t, path) in targets and t.has_action(t.ACTION_MOTION)):
                 tx, ty = t.get_pointer()
-                #index = targets.index(t)
-                #targetpath = path[:index + 1]
-                self.__call_sensor(t.get_action_call(t.ACTION_MOTION),
+                self._call_sensor(t.get_action_call(t.ACTION_MOTION),
                                    path, tx, ty)
 
         #end for
 
         self.__last_targets = targets
 
-        # save the window position if the window has moved
-        x, y = self.window.get_position()
-        if ((x, y) != self.__window_pos and not is_leave):
-            nil, nil, w, h = self.__group.get_geometry()
-            ax, ay = self.__group.get_anchored_coords(x, y, w, h)
-            dx, dy = x - ax, y - ay
-            self.__call_sensor([(DEFAULT_SENSOR, "move", [])], [""],
-                               x + dx, y + dy, x, y)
-
-            self.__window_pos = (x, y)
-            #print "save window @", x, y
-            self.__group.set_position(x + dx, y + dy, update = 0)
-
 
 
     #
@@ -382,6 +287,8 @@ class Display(GlassWindow, Observable):
     #
     def __on_scroll(self, src, event):
 
+        if (not self.__is_sensitive): return
+
         px, py = self.get_pointer()
         targets = self.__get_target_at(px, py)
 
@@ -396,9 +303,7 @@ class Display(GlassWindow, Observable):
         for t, path in targets:
             if (t.has_action(DisplayTarget.ACTION_SCROLL)):
                 call = t.get_action_call(DisplayTarget.ACTION_SCROLL)
-                #index = targets.index(t)
-                #targetpath = path[:index + 1]
-                self.__call_sensor(call, path, direction)
+                self._call_sensor(call, path, direction)
                 break
         #end for
     
@@ -411,39 +316,15 @@ class Display(GlassWindow, Observable):
 
         # propagate the incoming sensor output
         if (cmd == src.OBS_OUTPUT):
-            #if (self.window): self.window.freeze_updates()
-            for key, value in data.get_entries():
-                if ("[" in key):
-                    indexpart = key[key.find("[") + 1:-1]
-                    indexes = indexpart.split("][")
-                    key = key[:key.find("[")]
-                else:
-                    indexes = []
-
-                entries = self.__mapping.get((src, key), [])
-                for target, property in entries:
-                    if (indexes):
-                        sensor = self.__get_sensor_id(src)
-                        target.distribute_sensor_output(sensor, indexes[:],
-                                                        key, value)
-                    else:
-                        target.set_config(property, value)
-                #end for
-            #end for
-            #if (self.window): self.window.thaw_updates()
-
-            # is this queue_draw() needed? this was the CPU hog
-            #self.queue_draw()
+            self.__set_settings(src, data)
 
         elif (cmd == src.OBS_CMD_CONFIGURE):
             self.__open_configurator()
 
         elif (cmd == src.OBS_CMD_REMOVE):
-            #self.__remove_display(restart = 0)
             self.update_observer(self.OBS_CLOSE, self.__id)
 
         elif (cmd == src.OBS_CMD_RESTART):
-            #self.__remove_display(restart = 1)
             self.update_observer(self.OBS_RESTART, self.__id)
 
         elif (cmd == src.OBS_CMD_DUPLICATE):
@@ -457,36 +338,29 @@ class Display(GlassWindow, Observable):
     #
     def __on_observe_group(self, src, cmd, *args):
 
-        if (not self.__is_dragging and cmd == src.OBS_MOVE):
+        if (cmd == src.OBS_MOVE):
             x, y, w, h = args
+            ux, uy, nil, nil = self.__group.get_user_geometry()
             ax, ay = self.__group.get_anchored_coords(x, y, w, h)
-
-            ax = min(max(0, ax), gtk.gdk.screen_width())
-            ay = min(max(0, ay), gtk.gdk.screen_height())
-
-            if ((ax, ay) == (0, 0) or (ax, ay) != self.__window_pos):
-                self.move(ax, ay)
-                self.__window_pos = (ax, ay)
-                self.__call_sensor([(DEFAULT_SENSOR, "positioned", [])], [""])
+            if (ux == uy == UNSET_COORD):
+                self.__container.set_position(ux, uy)
+            else:
+                self.__container.set_position(ax, ay)
+            self._call_sensor([(DEFAULT_SENSOR, "positioned", [])], [""])
                 
-
             if (w != 0 and h != 0):
-                self.resize(w, h)
+                # TODO: set the display size instead of the window size
+                self.__container.resize(w, h)
                 # storing the size is useless, but it's added by request;
                 # it makes life easy for desklets pagers
-                self.__call_sensor([(DEFAULT_SENSOR, "size", [])], [""], w, h)
-
-            # don't show this window before its position has been set
-            if (self.__window_pos != (-1, -1)):
-                gtk.idle_add(self.show)
-                gtk.idle_add(self.move, ax, ay)
+                self._call_sensor([(DEFAULT_SENSOR, "size", [])], [""], w, h)
 
 
 
     #
     # Calls a function of a Sensor.
     #
-    def __call_sensor(self, cmd, path, *args):
+    def _call_sensor(self, cmd, path, *args):
         assert(cmd)
 
         args = list(args)
@@ -517,6 +391,48 @@ class Display(GlassWindow, Observable):
         return targets
 
 
+    #
+    # Saves the the given position of the display. The position is nw-anchored.
+    #
+    def save_position(self, x, y):
+
+        nil, nil, w, h = self.__group.get_geometry()
+        ax, ay = self.__group.get_anchored_coords(x, y, w, h)
+        dx, dy= x - ax, y - ay
+        self._call_sensor([(DEFAULT_SENSOR, "move", [])], [""],
+                           x + dx, y + dy, x, y)
+        self.__group.set_position(x, y, update = 0)
+    
+
+    #
+    # Sets the configuration settings.
+    #
+    def __set_settings(self, sensor, settings):
+
+        for key, value in settings.get_entries():
+            # extract array indexes
+            if ("[" in key):
+                indexpart = key[key.find("[") + 1:-1]
+                indexes = indexpart.split("][")
+                key = key[:key.find("[")]
+            else:
+                indexes = []
+
+            # get all (target, property) tuples that are watching the given
+            # sensor key and notify the targets
+            entries = self.__mapping.get((sensor, key), [])
+            for target, property in entries:
+                if (indexes):
+                    # special handling for arrays
+                    sensor_id = self.__get_sensor_id(sensor)
+                    target.distribute_sensor_output(sensor_id, indexes[:],
+                                                    key, value)
+                else:
+                    target.set_config(property, value)
+            #end for
+        #end for
+
+
 
     #
     # Sets the configuration.
@@ -524,10 +440,7 @@ class Display(GlassWindow, Observable):
     def set_config(self, key, value):
 
         if (key == "window-flags"):
-            flags = 0
-            value = value.split(",")
-            for p in value: flags |= self.__WINDOW_FLAGS[p.strip()]
-            self.set_window_type(flags)
+            self.__container.set_window_flags(value)
 
         elif (key == "shape"):
             file = vfs.join(self.get_path(), value)
@@ -539,12 +452,11 @@ class Display(GlassWindow, Observable):
                 loader.write(data, len(data))
                 loader.close()
                 pixbuf = loader.get_pixbuf()
-                #pixbuf = gtk.gdk.pixbuf_new_from_file(file)
                 pix, mask = pixbuf.render_pixmap_and_mask(1)
-                self.shape_combine_mask(mask, 0, 0)
+                self.__container.set_shape(mask)
             except:
                 print "could not set shape", file
-
+        
         else:
             self.__group.set_config(key, value)
 
