@@ -1,6 +1,7 @@
 from DataTarget import DataTarget
 from utils.datatypes import *
 from urllib import unquote # for filename conversion
+from main import UNSET_COORD
 import gtk
 
 
@@ -174,7 +175,6 @@ class DisplayTarget(gtk.HBox, DataTarget):
             is_px, is_py = self.__is_percent_coords
 
             if (is_percent): self.__is_percent_coords = (1, is_py)
-            self.__user_coords = (v, y)
             self.set_position(v, y)
 
         elif (key == "y"):
@@ -183,7 +183,6 @@ class DisplayTarget(gtk.HBox, DataTarget):
             is_px, is_py = self.__is_percent_coords
 
             if (is_percent): self.__is_percent_coords = (is_px, 1)
-            self.__user_coords = (x, v)
             self.set_position(x, v)
 
         elif (key == "width"):
@@ -264,6 +263,8 @@ class DisplayTarget(gtk.HBox, DataTarget):
     #
     def get_anchored_coords(self, x, y, w, h):
 
+        if (x == y == UNSET_COORD): return (x, y)
+
         anchor = self.__anchor
         if (anchor in [self.ANCHOR_NW, self.ANCHOR_W, self.ANCHOR_SW]):
             ax = x
@@ -297,6 +298,8 @@ class DisplayTarget(gtk.HBox, DataTarget):
     #
     def set_position(self, x, y, update = 1):
 
+        self.__user_coords = (x, y)
+
         # handle relative positioning
         if (self.__relative):
             is_rel_x, is_rel_y = self.__is_relative
