@@ -28,7 +28,7 @@ class TargetBonoboControl(DisplayTarget):
 
         value = self._convert_type(key, value)
 
-        if (key == "oafiid" and not self.__control):
+        if (key == "oafiid"):
             try:
                 container = bonobo.ui.Container()
                 control = bonobo.ui.Widget(str(value),
@@ -38,7 +38,12 @@ class TargetBonoboControl(DisplayTarget):
                 print "Property Bag:\n", slots
                 
                 control.show()
+
+                if self.__control:  # we have to remove the previous control
+                    self.remove( self.__control )
+
                 self.add(control)
+                self.__control = control
                 
             except StandardError, e:
                 print e
