@@ -1,5 +1,6 @@
 from DisplayTarget import DisplayTarget
 from utils.datatypes import *
+from utils import vfs
 
 import gtk
 import os
@@ -26,6 +27,10 @@ class TargetImage(DisplayTarget):
         # the size values
         self.__width = 0
         self.__height = 0
+        self.__original_width = 0
+        self.__original_height = 0
+        self.__current_width = 0
+        self.__current_height = 0
         self.__scale = 1.0
         
         # image cache
@@ -55,20 +60,27 @@ class TargetImage(DisplayTarget):
         value = self._convert_type(key, value)
 
 
-        if (key == "uri"): self.__load_image(value)
-        elif (key == "scale"): self.__scale_image(value)
+        if (key == "uri"):
+            try:
+                self.__load_image(value)
+            except StandardError, e:
+                import traceback; traceback.print_exc()
+                print e
+                
+        elif (key == "scale" and abs(value - self.__scale) > 0.01):
+            self.__scale_image(value)
 
-        elif (key == "image-width"):
+        elif (key == "image-width" and value != self.__width):
             self.__width = value
             self.__render_image(self.__width, self.__height)
             
-        elif (key == "image-height"):
+        elif (key == "image-height" and value != self.__height):
             self.__height = value
             self.__render_image(self.__width, self.__height)
 
-        elif (key == "saturation"):
+        elif (key == "saturation" and abs(value - self.__saturation) > 0.01 ):
             self.__saturation = value
-            #self.__render_image(self.__width, self.__height)
+            self.__render_image()
 
         else:
             DisplayTarget.set_config(self, key, value)
@@ -78,35 +90,40 @@ class TargetImage(DisplayTarget):
 
     #
     # Loads the image from the given URI.
-    # TODO: use GNOME VFS
     #
     def __load_image(self, uri):
 
-        uri = os.path.join(self.get_path(), uri)
+        uri = vfs.join(self.get_path(), uri)
         if (uri != self.__uri):
-            self.__original_image = gtk.gdk.pixbuf_new_from_file(uri)
+            loader = gtk.gdk.PixbufLoader()
+            fd = vfs.open(uri, "r")
+            data = vfs.read_all(fd)
+            fd.close()
+            loader.write(data, len(data))
+            loader.close()
+            self.__original_image = loader.get_pixbuf()
+            #self.__original_image = gtk.gdk.pixbuf_new_from_file(uri)
             self.__widget.set_from_pixbuf(self.__original_image)
             self.__uri = uri
             self.__cache = {}
         #end if
 
         if (self.__original_image):
-            if (not self.__width):
-                self.__width= self.__original_image.get_width()
-            if (not self.__height):
-                self.__height = self.__original_image.get_height()
-            self.__render_image()
+            w = self.__original_width = self.__original_image.get_width()
+            h = self.__original_height = self.__original_image.get_height()
+            self.__scale_image(self.__scale)
         #end if
 
 
 
+
     #
     # Scales the image by the given factor.
     #
     def __scale_image(self, scale):
 
-        width = int(self.__width * scale)
-        height = int(self.__height * scale)
+        width = int((self.__width or self.__original_width) * scale)
+        height = int((self.__height or self.__original_height) * scale)
         self.__render_image(width, height)
             
         self.__scale = scale
@@ -115,27 +132,21 @@ class TargetImage(DisplayTarget):
 
     #
     # Renders the image at the given size. Whithout size arguments,
-    # the original size is used.
+    # the current size is used.
     #
     def __render_image(self, width = 0, height = 0):
 
         if (not self.__original_image): return
 
-        w = self.__width #self.__original_image.get_width()
-        h = self.__height #self.__original_image.get_height()
-
-        if ((w, h) != (width, height) and width != 0 and height != 0):
-            key = (width, height)
-            if (_USE_CACHE and self.__cache.has_key(key)):
-                img = self.__cache[key]
-            else:
-                img = self.__original_image.scale_simple(width, height, 3)
-                if (_USE_CACHE): self.__cache[key] = img
-                
+        w = width or self.__current_width
+        h = height or self.__current_height
+        
+        key = (w, h)
+        if (_USE_CACHE and self.__cache.has_key(key)):
+            img = self.__cache[key]
         else:
-            #img = self.__original_image
-            img = self.__original_image.scale_simple(self.__width,
-                                                     self.__height, 3)
+            img = self.__original_image.scale_simple(w, h, 3)
+            if (_USE_CACHE): self.__cache[key] = img
 
         if (abs(self.__saturation - 1.0) > 0.00001):
             simg = img.copy()
@@ -144,3 +155,5 @@ class TargetImage(DisplayTarget):
             simg = img
 
         self.__widget.set_from_pixbuf(simg)
+        self.__current_width = w
+        self.__current_height = h
