@@ -16,6 +16,7 @@ class TargetLabel(DisplayTarget):
         DisplayTarget.__init__(self, parent, display)
 
         self.__widget = gtk.Label("")
+        self.__widget.set_line_wrap(gtk.FALSE)
         self.__widget.show()
 
         self.add(self.__widget)
@@ -23,6 +24,7 @@ class TargetLabel(DisplayTarget):
         self._set_property_type("value", TYPE_STRING)
         self._set_property_type("color", TYPE_STRING)
         self._set_property_type("font", TYPE_STRING)
+        self._set_property_type("wrap-at", TYPE_INT)
 
 
 
@@ -43,6 +45,16 @@ class TargetLabel(DisplayTarget):
         self.__old_value = ""
 
 
+    def __set_wrap(self, value):
+
+        if (value == 0):
+            self.__widget.set_size_request(-1, -1)
+            self.__widget.set_line_wrap(gtk.FALSE)
+        else:
+            self.__widget.set_size_request(value, -1)
+            self.__widget.set_line_wrap(gtk.TRUE)
+            
+
 
     def set_config(self, key, value):
 
@@ -70,6 +82,8 @@ class TargetLabel(DisplayTarget):
             self.__set_font(value)
         elif (key == "color"):
             self.__set_color(value)
+        elif (key == "wrap-at"):
+            self.__set_wrap(value)
         else:
             DisplayTarget.set_config(self, key, value)
 
