@@ -8,6 +8,7 @@ from utils import singleton
 from utils.TypeConverter import TypeConverter
 from utils.datatypes import *
 from utils import dialog
+from config import settings as cfgsettings
 
 from main import _
 from main import DEFAULT_SENSOR
@@ -53,6 +54,8 @@ class DisplayFactory(sax.handler.ContentHandler):
             print "Parse Error:", e
             # abort if an parse error occured
             return None
+        except:
+            return None
 
         # add the sensors
         for id, sensor in self.__sensors:
@@ -60,6 +63,7 @@ class DisplayFactory(sax.handler.ContentHandler):
 
         # add the DefaultSensor
         defsensor = DefaultSensor()
+        defsensor.set_profile(cfgsettings.profile)
         defsensor.set_config_id(self.__id + DEFAULT_SENSOR)
         self.__display.add_sensor(DEFAULT_SENSOR, defsensor)
 
@@ -84,11 +88,11 @@ class DisplayFactory(sax.handler.ContentHandler):
             # make the corrupt display visible so that the user can
             # remove it
             broken_display = self.__display.get_path()
-            self.__display.show()
-            self.__display.set_decorated(1)
+            #self.__display.show()
+            #self.__display.set_decorated(1)
             self.__display.set_size_request(500, 100)
-            self.__display.set_title("Broken display in %s" %broken_display )
-            self.__display.set_transparency_mode(0, 500, 100)
+            #self.__display.set_title("Broken display in %s" %broken_display )
+            #self.__display.set_transparency_mode(0, 500, 100)
             print "Error while configuring display:" + str(e)
             dialog.warning(_("Error while configuring display"),
                            _("A display could not be configured properly. "
@@ -98,7 +102,7 @@ class DisplayFactory(sax.handler.ContentHandler):
         dsp = self.__display
         self.__display = None
         self.__sensors = []
-        
+
         return dsp
 
 
@@ -132,9 +136,13 @@ class DisplayFactory(sax.handler.ContentHandler):
             args = moduledata[1:]
             sensor = self.__sensor_factory.create_sensor(module, args)
             if (sensor):
+                sensor.set_profile(cfgsettings.profile)
                 sensor.set_id(id)
                 sensor.set_config_id(self.__id + id)
                 self.__sensors.append((id, sensor))
+            else:
+                raise RuntimeError("Could not load sensor")
+
 
         # ignore the <meta> tag for now
         elif (name == "meta"):
