@@ -7,7 +7,8 @@ class Disk:
     def __init__(self):
 
         self.get_partitions = polling.wrap(self.__poll_partitions, 3)
-        self.get_size = self.__poll_size
+        self.get_size       = polling.wrap(self.__poll_size, 3)
+        self.get_fs         = polling.wrap(self.__poll_fs, 3)
 
 
 
@@ -35,3 +36,13 @@ class Disk:
         used = total - free
 
         return (total, used)
+
+
+
+    def __poll_fs(self, partition):
+
+        data = glibtop.get_mountlist(0)
+        partitions = []
+        for size, device, mpoint, fstype in data:
+            if (mpoint == partition):
+                return fstype
