@@ -1,9 +1,11 @@
 from main import USERHOME, HOME, NAME, PURGE_KEY
 from factory.DisplayFactory import DisplayFactory
+from display.Window import Window
 from config.ConfigManager import ConfigManager
 from utils import singleton
 from utils import dialog
 from utils import vfs
+from config import settings
 import admin
 
 from main import _
@@ -23,38 +25,7 @@ class Starter:
         self.__factory = singleton.get(DisplayFactory)
         self.__config = singleton.get(ConfigManager)
 
-
-        # watches main/displays in the configuration base and calls on_watch if
-        # something's changed
-        self.__config.watch("main", "displays", self.__on_watch)
-
-        # gets current displays from the configuration base
-        displays = self.__config.get("main", "displays")
-
-        # check whether there are already some displays; if there aren't any
-        # welcome new users
-        if (not displays): self.__first_run()
-
-
-
-    #
-    # Welcomes new users and sets up the configuration base.
-    #
-    def __first_run(self):
-
-        name = "<i>" + NAME + "</i>"
-        dialog.info(_("Welcome to %(name)s<i>!</i>") % vars(),
-                    _("It looks like you are running %(name)s for the first "
-                      "time.\n"
-                      "\n"
-                      "You can add new displays by launching them in your "
-                      "file manager.\n"
-                      "\n"
-                      "<b>Have fun,</b>\n"
-                      "  <i>the %(name)s developers</i>") % vars())
-
-        self.__config.set("main", "displays", "")
-
+        self.__config.watch("profile", self.__on_change_profile)
 
 
     #
@@ -76,6 +47,26 @@ class Starter:
 
 
 
+    #
+    # Reacts on changing the user profile.
+    #
+    def __on_change_profile(self, key, value):
+
+        if (settings.profile == value): return
+        
+        # shut down displays
+        for id, d in self.__open_displays.items():
+            d.remove_display()
+            del self.__open_displays[id]
+
+        settings.profile = value
+        # load displays
+        self.start_displays()
+        print "PROFILE CHANGE", value
+        pass
+        
+
+
     #
     # Reacts on changes in the list of displays.
     #
@@ -96,7 +87,6 @@ class Starter:
         #end for
 
 
-
     #
     # Adds the given display.
     #
@@ -107,9 +97,13 @@ class Starter:
             self.__open_displays[id] = dsp
 
             if (dsp):
-                print "adding", id, "\n[" + path + "]"
+                print "[" + path + "]"
                 self.__open_displays[id] = dsp
                 dsp.add_observer(self.__on_display_action)
+                win = Window(dsp)
+                win.move(-2000, -2000)
+                win.show()
+                dsp.show()
         #end if
 
 
@@ -177,7 +171,7 @@ class Starter:
     #
     def __purge_config(self):
 
-        for dir in self.__config.list():
+        for dir in self.__config.list("profiles", settings.profile):
             if (not dir[0].isdigit()): continue
 
             to_remove = 1
@@ -189,7 +183,8 @@ class Starter:
 
             if (to_remove):
                 # get paths to purge
-                purge = self.__config.get(dir, PURGE_KEY).split(",")
+                purge = self.__config.get("profiles", settings.profile,
+                                          dir, PURGE_KEY).split(",")
                 for p in purge:
                     if (not p): continue
 
@@ -203,7 +198,7 @@ class Starter:
                 #end for
                 
                 print "purging", dir
-                self.__config.clear(dir)
+                self.__config.clear("profiles", settings.profile, dir)
         #end for
 
 
@@ -213,5 +208,11 @@ class Starter:
     #
     def start_displays(self):
 
-        displays = self.__config.get("main", "displays")
+        # watches main/displays in the configuration base and calls on_watch if
+        # something's changed
+        self.__config.watch("profiles", settings.profile, "main", "displays",
+                            self.__on_watch)
+
+        displays = self.__config.get("profiles", settings.profile,
+                                     "main", "displays")
         self.__on_watch(None, None)
