@@ -13,6 +13,8 @@ PURGE_KEY = "_paths_to_purge_"
 # the ID of the default sensor
 DEFAULT_SENSOR = "_default_"
 
+UNSET_COORD = -1000
+
 # paths where gDesklets looks for sensors and displays
 SENSORPATHS = [os.path.join(USERHOME, "Sensors"),
                os.path.join(HOME, "Sensors")]
@@ -20,7 +22,7 @@ DISPLAYPATHS = [os.path.join(USERHOME, "Displays"),
                 os.path.join(HOME, "Displays")]
 
 NAME = "gDesklets"
-VERSION = "0.24.1"
+VERSION = "0.25"
 COPYRIGHT = u"Copyright \xa9 2003 " \
             "Martin Grimme, Christian Meyer, Jesse Andrews"
 DESCRIPTION = u"A desktop applet system for GNOME"
@@ -32,26 +34,20 @@ AUTHORS = [u"Martin Grimme <martin@pycage.de>",
 DOCUMENTERS = [u"Martin Grimme <martin@pycage.de>"]
 ICON = os.path.join(HOME, "data", "gdesklets.png")
 
-
-
-# check python version
-#if (float(sys.version[:3]) < 2.3):
-#    print "Python 2.3 is recommended."
-#    print "Python " + (sys.version[:3]) + \
-#          " might result in unexpected behavior!\n"
-
-
-
 # setup GTK
 try:
     import pygtk; pygtk.require("2.0")
 except:
     print "You need a recent version of PyGTK to run this program."
     #import sys; sys.exit(1)
+
+
+
 import gnome.ui
 import gtk
 
 
+
 # init gnome
 chldhandler = signal.getsignal(signal.SIGCHLD)
 program = gnome.program_init(NAME, VERSION)
@@ -70,8 +66,6 @@ _ = Translator(NAME.lower(), os.path.join(HOME, "locale"))
 
 
 
-
-
 #
 # Make sure that EventBoxes are transparent.
 #
@@ -88,9 +82,10 @@ rc_style = """
 gtk.rc_parse_string(rc_style)
 
 
+def print_about():
 
-print NAME + " " + VERSION
-print COPYRIGHT.encode("Latin-1")
-print ""
-print "This software is licensed under the terms of the GNU GPL."
-print ""
+    print NAME + " " + VERSION
+    print COPYRIGHT.encode("Latin-1")
+    print ""
+    print "This software is licensed under the terms of the GNU GPL."
+    print ""
