@@ -13,15 +13,13 @@ class Observable:
             self.__handlers
         except:
             self.__handlers = []
-            self.__len = 0
 
-        
+
 
     def add_observer(self, observer):
 
         self.__ensure_init()
         self.__handlers.append(observer)
-        self.__len += 1
 
 
 
@@ -31,6 +29,14 @@ class Observable:
         self.__handlers.remove(observer)
 
 
+
+    def drop_observers(self):
+
+        self.__ensure_init()
+        self.__handlers = []
+
+
+
     def update_observer(self, *args):
 
         self.__ensure_init()
